#!/usr/bin/perl -w

require '/var/smoothwall/header.pl';

my %dhcpsettings;
my %netsettings;

print "Content-type: text/html\n\n";

$netsettings{'DRIVER'} = '';
$netsettings{'IPADDR'} = '';
$netsettings{'NETADDR'} = '';
$netsettings{'NETMASK'} = '';
&readhash('/etc/sysconfig/network', \%netsettings);

$dhcpsettings{'ACTION'} = '';
$dhcpsettings{'VALID'} = '';

$dhcpsettings{'ENABLE'} = 'off';
$dhcpsettings{'START_ADDR'} = '';
$dhcpsettings{'END_ADDR'} = '';
$dhcpsettings{'DNS1'} = '';
$dhcpsettings{'DNS2'} = '';
$dhcpsettings{'DOMAIN_NAME'} = '';
$dhcpsettings{'DEFAULT_LEASE_TIME'} = '';
$dhcpsettings{'MAX_LEASE_TIME'} = '';

&getcgihash(\%dhcpsettings);

$errormessage = '';
if ($dhcpsettings{'ACTION'} eq 'Save')
{ 
	if (!($dhcpsettings{'START_ADDR'} =~ /^\d+\.\d+\.\d+\.\d+$/))
	{
		$errormessage = 'Invalid start address.';
		goto ERROR;
	}
	if (!($dhcpsettings{'END_ADDR'} =~ /^\d+\.\d+\.\d+\.\d+$/))
	{
		$errormessage = 'Invalid end address.';
		goto ERROR;
	}
	if ($dhcpsettings{'DNS1'})
	{
		if (!($dhcpsettings{'DNS1'} =~ /^\d+\.\d+\.\d+\.\d+$/))
		{
			$errormessage = 'Invalid primary DNS.';
			goto ERROR;
		}
	}
	if ($dhcpsettings{'DNS2'})
	{
		if (!($dhcpsettings{'DNS2'} =~ /^\d+\.\d+\.\d+\.\d+$/))
		{
			$errormessage = 'Invalid secondary DNS.';
			goto ERROR;
		}
	}
	if (!($dhcpsettings{'DNS1'}) && $dhcpsettings{'DNS2'})
	{
		$errormessage = 'Cannot specify secondary DNS without specifying primary.'; 
		goto ERROR;
	}
	if (!($dhcpsettings{'DEFAULT_LEASE_TIME'} =~ /^\d+$/))
	{
		$errormessage = 'Invalid default lease time.';
		goto ERROR;
	}
	if (!($dhcpsettings{'MAX_LEASE_TIME'} =~ /^\d+$/))
	{
		$errormessage = 'Invalid max lease time.';
		goto ERROR;
	}
ERROR:
	if ($errormessage) {
		$dhcpsettings{'VALID'} = 'no'; }
	else {
              	$dhcpsettings{'VALID'} = 'yes'; }

	&writehash("${swroot}/dhcp/settings", \%dhcpsettings);
	open(FILE, ">/${swroot}/dhcp/dhcpd.conf") or die "Unable to write dhcpd.conf file";
	print FILE "subnet $netsettings{'NETADDR'} netmask $netsettings{'NETMASK'}\n";
	print FILE "{\n";
	print FILE "\toption subnet-mask $netsettings{'NETMASK'};\n";
	print FILE "\toption domain-name \"$dhcpsettings{'DOMAIN_NAME'}\";\n";
	print FILE "\toption routers $netsettings{'IPADDR'};\n";
	if ($dhcpsettings{'DNS1'})
	{
		print FILE "\toption domain-name-servers ";
		print FILE "$dhcpsettings{'DNS1'}";
		if ($dhcpsettings{'DNS2'}) {
			print FILE ", $dhcpsettings{'DNS2'}"; }
		print FILE ";\n";
	}
	my $defaultleasetime = $dhcpsettings{'DEFAULT_LEASE_TIME'} * 60;
	my $maxleasetime = $dhcpsettings{'MAX_LEASE_TIME'} * 60;
	print FILE "\trange dynamic-bootp $dhcpsettings{'START_ADDR'} $dhcpsettings{'END_ADDR'};\n";
	print FILE "\tdefault-lease-time $defaultleasetime;\n";
	print FILE "\tmax-lease-time $maxleasetime;\n";
	print FILE "}\n";
	close FILE;

	if ($dhcpsettings{'ENABLE'} eq 'on' && $dhcpsettings{'VALID'} eq 'yes')
	{
		system ('/bin/touch', "${swroot}/dhcp/enable");
		&log('DHCP server enabled.  Restarting.')
	}
	else
	{
		unlink "${swroot}/dhcp/enable";
		&log('DHCP server disabled.  Stopping.')
	}		
		
	system '/usr/local/bin/restartdhcp';
}

&readhash("${swroot}/dhcp/settings", \%dhcpsettings);

if ($dhcpsettings{'VALID'} eq '')
{
 	$dhcpsettings{'ENABLE'} = 'off';
        $dhcpsettings{'DNS1'} = $netsettings{'IPADDR'};
        $dhcpsettings{'DEFAULT_LEASE_TIME'} = '60';
        $dhcpsettings{'MAX_LEASE_TIME'} = '120';
}

$checked{'ENABLE'}{'off'} = '';
$checked{'ENABLE'}{'on'} = '';
$checked{'ENABLE'}{$dhcpsettings{'ENABLE'}} = 'CHECKED';

&openpage('DHCP configuration', 1);

&openbigbox('100%', 'LEFT');

print "<FORM METHOD='POST'>\n";

&openbox('100%', 'LEFT', 'DHCP:');
print <<END
<TABLE WIDTH='100%'>
<TR>
	<TD>Start address:</TD>
	<TD><INPUT TYPE='text' NAME='START_ADDR' VALUE='$dhcpsettings{'START_ADDR'}'></TD>
	<TD>End address:</TD>
	<TD><INPUT TYPE='text' NAME='END_ADDR' VALUE='$dhcpsettings{'END_ADDR'}'></TD>
</TR>
<TR>
	<TD>Primary DNS:</TD>
	<TD><INPUT TYPE='text' NAME='DNS1' VALUE='$dhcpsettings{'DNS1'}'></TD>
	<TD>Secondary DNS:</TD>
	<TD><INPUT TYPE='text' NAME='DNS2' VALUE='$dhcpsettings{'DNS2'}'></TD>
</TR>
<TR>
	<TD>Default lease time (mins):</TD>
	<TD><INPUT TYPE='text' NAME='DEFAULT_LEASE_TIME' VALUE='$dhcpsettings{'DEFAULT_LEASE_TIME'}'></TD>
	<TD>Max lease time (mins):</TD>
	<TD><INPUT TYPE='text' NAME='MAX_LEASE_TIME' VALUE='$dhcpsettings{'MAX_LEASE_TIME'}'></TD>
</TR>
<TR>
	<TD>Domain name suffix:</TD>
	<TD><INPUT TYPE='text' NAME='DOMAIN_NAME' VALUE='$dhcpsettings{'DOMAIN_NAME'}'></TD>
	<TD>Enabled:</TD>
	<TD><INPUT TYPE='checkbox' NAME='ENABLE' $checked{'ENABLE'}{'on'}></TD>
</TR>

</TABLE>
END
;
&closebox();

&openbox('100%', 'LEFT', 'Error messages:');
print "$errormessage\n";
print "&nbsp;\n";
&closebox();

print <<END
<DIV ALIGN='CENTER'>
<TABLE WIDTH='60%'>
<TR>
	<TD ALIGN='CENTER'><INPUT TYPE='submit' NAME='ACTION' VALUE='Save'></TD>
	<TD ALIGN='CENTER'><A HREF='/cgi-bin/index.cgi'>BACK</A></TD>
	<TD ALIGN='CENTER'><A HREF='/configure.html#3.4.'>HELP</A></TD>
</TR>
</TABLE>
</DIV>
END
;

print "</FORM>\n";

&closebigbox();

&closepage();
